@@ -6,12 +6,14 @@
 #' @param iv Name of independent variable
 #' @param mediators Name of mediators
 #' @param dv Name of dependent variable
+#' @param covariates Name of covariates to be included in model.
 #' @param type What type of penalty. Options include lasso, ridge, and enet.
 #' @param nfolds Number of cross-validation folds.
 #' @param epsilon Threshold for determining whether effect is 0 or not.
 #' @param seed Set seed to control CV results
 #' @export
 #' @examples
+#' \dontrun{
 #'# example
 #'library(ISLR)
 #'College1 = College[which(College$Private=="Yes"),]
@@ -48,9 +50,11 @@
 #'
 #'out <- xmed_cat(Data,iv,mediators,dv)
 #'out
+#'}
 
 
-xmed_cat = function (data, iv, mediators, dv, type = "lasso", nfolds = 10, epsilon = 0.001, seed = NULL)
+xmed_cat = function (data, iv, mediators, dv, covariates=NULL, type = "lasso", nfolds = 10,
+                     epsilon = 0.001, seed = NULL)
 {
   res <- list()
   Data <- data
